/*
    Copyright (C) 2016-2018 Chih-Hsuan Yen <yan12125@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef TOUCHPADCONFIG_H
#define TOUCHPADCONFIG_H

#include <QWidget>
#include "ui_touchpadconfig.h"

namespace LXQt
{
    class Settings;
}

class TouchpadDevice;

class TouchpadConfig : public QWidget
{
    Q_OBJECT

public:
    TouchpadConfig(LXQt::Settings* _settings, QWidget* parent);
    virtual ~TouchpadConfig();

    void accept();
    void applyConfig();

public Q_SLOTS:
    void reset();

Q_SIGNALS:
    void settingsChanged();

private:
    void initControls();
    void initFeatureControl(QCheckBox* control, int featureEnabled);

private:
    LXQt::Settings* settings;
    Ui::TouchpadConfig ui;
    QList<TouchpadDevice> devices;
};

#endif
