/*
    Copyright (C) 2016  P.L. Lucas <selairi@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef _FASTMENU_H_
#define _FASTMENU_H_

#include "ui_fastmenu.h"

#include <QGroupBox>
#include <QStringList>
#include <QHash>
#include <QList>
#include <KScreen/Config>



class FastMenu : public QGroupBox
{
  Q_OBJECT


public:
    FastMenu(KScreen::ConfigPtr config, QWidget* parent = nullptr);
    ~FastMenu() override;

private Q_SLOTS:
    void onSelectionChanged(int index);

private:
    Ui::FastMenu ui;
    KScreen::ConfigPtr mConfig;
    KScreen::ConfigPtr mOldConfig;

    void unified();
    void onlyFirst();
    void onlySecond();
};

#endif // _FASTMENU_H_
