/*
* Copyright (c) 2015-2024 LXQt Team.
*
* This file is part of the LXQt project. <https://lxqt-project.org>
* It is distributed under the LGPL 2.1 or later license.
* Please refer to the LICENSE file for a copy of the license, and
* the AUTHORS file for copyright and authorship information.
*/

#ifndef LXQT_CONFIG_INPUT_H
#define LXQT_CONFIG_INPUT_H



#endif // LXQT_CONFIG_INPUT_H
